<?php

/**
 * Initialization file for the Gitweb extension.
 *
 * Documentation:	 		https://www.mediawiki.org/wiki/Extension:Gitweb
 * Support					https://www.mediawiki.org/wiki/Extension_talk:Gitweb
 * Source code:				https://gerrit.wikimedia.org/r/gitweb?p=mediawiki/extensions/Gitweb.git
 *
 * @file
 * @ingroup Gitweb
 *
 * @licence GNU GPL v2+
 * @author Jeroen De Dauw < jeroendedauw@gmail.com >
 */

/**
 * This documentation group collects source code files belonging to the Gitweb extension.
 *
 * @defgroup Gitweb Gitweb
 */

namespace {

	if ( !defined( 'MEDIAWIKI' ) ) {
		die( 'Not an entry point.' );
	}

	define( 'Gitweb_VERSION', '0.2.0' );

	$wgExtensionCredits['other'][] = array(
		'path' => __FILE__,
		'name' => 'Gitweb',
		'version' => Gitweb_VERSION,
		'author' => array(
			'[https://www.mediawiki.org/wiki/User:Jeroen_De_Dauw Jeroen De Dauw]',
		),
		'url' => 'https://www.mediawiki.org/wiki/Extension:Gitweb',
		'descriptionmsg' => 'gitweb-desc'
	);

	$wgMessagesDirs['Gitweb'] = __DIR__ . '/i18n';
	$wgExtensionMessagesFiles['GitwebMagic'] = __DIR__ . '/Gitweb.i18n.magic.php';

}

namespace Gitweb {

	$egGitwebRoot = 'https://gerrit.wikimedia.org/r/gitweb';
	$egGitwebFile = 'README';
	$egGitwebRepo = 'mediawiki/core';
	$egGitwebCacheTtl = 60;

	$wgHooks['ParserFirstCallInit'][] = function( \Parser &$parser ) {
		$parser->setFunctionHook( 'gitweb', function( \Parser $parser, $file = '', $repo = '' ) {
			$sourceUrl = str_replace(
				array( '$1', '$2' ),
				array(
					$repo === '' ? $GLOBALS['egGitwebRepo'] : $repo,
					$file === '' ? $GLOBALS['egGitwebFile'] : $file,
				),
				$GLOBALS['egGitwebRoot'] . '?p=$1.git;a=blob_plain;f=$2;hb=HEAD'
			);

			$cache = \ObjectCache::getLocalClusterInstance();
			$cacheKey = $cache->makeKey( __METHOD__, $sourceUrl );
			$cachedValue = $cache->get( $cacheKey );

			if ( is_string( $cachedValue ) ) {
				return $cachedValue;
			}
			else {
				$fetchedValue = \Http::get( $sourceUrl );
				$cache->set( $cacheKey, $fetchedValue, $GLOBALS['egGitwebCacheTtl'] );
				return $fetchedValue;
			}
		} );
		return true;
	};

}
